@@ -16,6 +16,7 @@
 #include "acfg.h"
 #include "lockable.h"
 #include "cleaner.h"
+#include "tcpconnect.h"
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -33,21 +34,32 @@
 #include <iostream>
 #include <list>
 
+#include "acsyscap.h"
+
 #define FUSE_USE_VERSION 25
 #include <fuse.h>
 
+#ifdef HAVE_DLOPEN
+#include <dlfcn.h>
+#endif
 
 #define HEADSZ 5000
 #ifndef MIN
 #define MIN(a,b) ( (a<=b)?a:b)
 #endif
 
+using namespace std;
+
+#ifdef SPAM
 #define _cerr(x) cerr << x
+#warning printing spam all around
+#else
+#define _cerr(x)
+#endif
 
 #define POOLMAXSIZE 20 // max size
 #define POOLMAXAGE 50 // seconds
 
-using namespace std;
 
 const time_t cleaner::never=ULONG_MAX;
 
@@ -62,30 +74,99 @@ void err(const char*, const char*)
 static struct stat statTempl;
 static struct statfs stfsTemp;
 static tHttpUrl baseUrl, proxyUrl;
-
-lockable mx;
-list< pair<time_t, int> > fdpool;
+static mstring altPath;
 
 struct tDlDesc
 {
-private:
-	header h;
-	int fd;
-public:
 	off_t seen_length;
 	int seen_status;
-	tDlDesc(): fd(-1), seen_length(0), seen_status(0)
-	{};
-	~tDlDesc() {
-		if(fd>=0)
-		{
-			lockguard g(mx);
-			fdpool.push_back(pair<time_t,int>(time(0),fd));
-		}
-	};
-	
 	acbuf databuf;
-	
+
+	//virtual bool CheckOrExplore(const char *path) =0;
+	virtual bool Act(const char *path, off_t pos, off_t len, bool bJustProbe=false) =0;
+
+	tDlDesc() : seen_length(0), seen_status(0) {};
+	virtual ~tDlDesc() {};
+};
+
+struct tDlDescLocal : public tDlDesc
+{
+	static tDlDesc* Create(const char *path)
+	{
+		tDlDescLocal *p(NULL);
+		MYTRY
+		{
+			p = new tDlDescLocal;
+			if (!p) // running exception-free?
+				return NULL;
+		}
+		MYCATCH(...)
+		{
+			return NULL;
+		}
+
+		FILE *pf = fopen(path, "rb");
+		if (!pf)
+		{
+			delete p;
+			return NULL;
+		}
+
+		p->seen_status = 200;
+		fseeko(pf, 0, SEEK_END);
+		p->seen_length = ftello(pf);
+
+		p->pFile = pf;
+
+		return p;
+	}
+	FILE *pFile;
+	tDlDescLocal() : pFile(NULL) {};
+	virtual ~tDlDescLocal()
+	{
+		if(pFile)
+			fclose(pFile);
+		pFile=NULL;
+	};
+
+	bool Act(const char *path, off_t pos, off_t len, bool bJustProbe=false)
+	{
+		databuf.clear();
+		if(databuf.freecapa() < len)
+			databuf.init(len);
+
+		if(pFile && 0==fseeko(pFile, pos, SEEK_SET))
+		{
+			while(!feof(pFile) && !ferror(pFile) && databuf.size() <len)
+			{
+				size_t r = ::fread(databuf.wptr(), 1, len-databuf.size(), pFile);
+				databuf.got(r);
+			}
+		}
+
+		if(databuf.size() == len || seen_length == (pos+databuf.size()))
+		{
+			seen_status=206;
+			return true;
+		}
+
+		return false;
+	}
+};
+
+struct tDlDescRemote : public tDlDesc
+{
+	header h;
+
+	tDlDescRemote()
+	{
+		// expire the caches every time, should not cost much anyway
+		tcpconnect::ExpireCache();
+		CAddrInfo::ExpireCache();
+	};
+	virtual ~tDlDescRemote()
+	{
+	}
 	
 	bool CheckOrExplore(const char *path)
 	{
@@ -117,6 +198,20 @@ public:
 		return true;
 	}
 	
+
+	inline bool WaitForResponse(int fd)
+	{
+	// prevent the thing from looping too fast before server
+		fd_set rfds;
+		FD_ZERO(&rfds);
+		FD_SET(fd, &rfds);
+		struct timeval tv;
+		tv.tv_sec = 5;
+		tv.tv_usec = 0;
+		return 0 != select(fd + 1, &rfds, NULL, NULL, &tv);
+	}
+
+
 	bool Act(const char *path, off_t pos, off_t len, bool bJustProbe=false)
 	{
 		_cerr( path << ", from: " << pos << " , " << len << "bytes\n");
@@ -126,25 +221,34 @@ public:
 		if(bJustProbe)
 			len=100; // fake something more sensible for now
 		
-		if(!_Connect(false))
+		mstring sErr;
+		bool bSecondHand=false;
+
+		tTcpHandlePtr con = tcpconnect::CreateConnected(proxyUrl.sHost, proxyUrl.sPort, sErr,
+				&bSecondHand);
+		if(!con)
 			return false;
 		
-		string req(len==0?"HEAD ":"GET ");
+		tSS req;
+		req << (bJustProbe?"HEAD ":"GET ");
 		
-		_cerr( "reqtype: " << req);
-		req+=baseUrl.ToURI()+path+" HTTP/1.1\r\nConnection: Keep-Alive\r\nUser-Agent: ACNGFS\r\nX-Original-Source: 42\r\n";
+		req << baseUrl.ToURI() << path <<
+				" HTTP/1.1\r\n"
+				"Connection: Keep-Alive\r\n"
+				"User-Agent: ACNGFS\r\n"
+				"X-Original-Source: 42\r\n";
+
 		if(!bJustProbe && len>0)
 		{
-			char buf[100];
-			sprintf(buf, "Range: bytes=%lu-%lu\r\n", (long unsigned)pos, (long unsigned) (pos+len-1));
-			req+=buf;
+			req << "Range: bytes=" << (long unsigned)pos << "-"
+					<< (long unsigned) (pos+len-1) << "\r\n";
 		}
-		req+="\r\n";
-		_cerr( "requesting: " << req );
+		req<<"\r\n";
+		_cerr( "requesting: " << req.rptr() );
 		
 		for(bool bRetried=false;!req.empty();)
 		{
-			int n=::send(fd, req.c_str(), req.size(), MSG_NOSIGNAL);
+			int n=::send(con->GetFD(), req.c_str(), req.size(), MSG_NOSIGNAL);
 			if(n<0)
 			{
 				if(EAGAIN==errno)
@@ -162,14 +266,15 @@ public:
 					return false;
 				
 				bRetried=true;
-				_ShutDown();
 				
-				if(!_Connect(true))
+				// restart the connection once
+				con->Disconnect();
+				if(!con->Connect(proxyUrl.sHost, proxyUrl.sPort, sErr))
 					return false;
 				
 				continue;
 			}
-			req.erase(0, n);
+			req.drop(n);
 		}
 		
 		//_cerr( "init buf for bytes: " << len);
@@ -180,22 +285,28 @@ public:
 		{
 			_cerr( "remaining: " << remaining <<endl);
 			if(remaining<=0)
-				return true;
+				goto com_done_con_idle;
 			
+			bool bHaveData=WaitForResponse(con->GetFD());
+
 			// drop all stuff read during precaching
 			if(bJustProbe && h.type!=h.INVALID)
 				databuf.clear();
 			
 			int nToRead=MIN(remaining,databuf.freecapa());
       //assert(nToRead>0);
-			int n=::recv(fd, databuf.wptr(), nToRead, 0);
+			int n=::recv(con->GetFD(), databuf.wptr(), nToRead, 0);
 			_cerr( "got: " << n << " errno: " << errno);
+
+			if(bHaveData && 0 == n) // remote host is disconnecting :-(
+				return false;
+
 			if(n<=0)
-			{ // no other toleratable errors should appear here
+			{ // no other tolerable errors should appear here
 				if(EINTR==errno || EAGAIN==errno)
 					continue;
 				//perror("ERROR: got <=0 from read: ");
-				_ShutDown();
+				con.reset();
 				return false;
 			}
 			databuf.got(n);
@@ -240,13 +351,16 @@ public:
 				if(200==seen_status)
 					seen_length=contlen;
 				
-				if(len==0) // just HEAD
-					return true;
+				if(bJustProbe) // just HEAD
+					goto com_done_con_idle;
 			}
 			
 		}
+		com_done_con_idle:
+		tcpconnect::RecycleIdleConnection(con);
 		return true;
 	};
+	/*
 	inline void _ShutDown()
 	{
 		_cerr("Disconnecting...\n");
@@ -261,7 +375,7 @@ public:
 	{
 		if(fd>=0)
 			return true;
-		
+
 		if(!bForceFresh)
 		{
 			lockguard g(mx);
@@ -283,7 +397,7 @@ public:
 				return true;
 			}
 		}
-		
+
 		signal(SIGPIPE, SIG_IGN);
 		string sErr;
     CAddrInfo::SPtr dns=CAddrInfo::CachedResolve(proxyUrl.sHost, proxyUrl.sPort, sErr);
@@ -310,52 +424,68 @@ public:
 		}
 		return true;
 	}
-
+*/
 };
 
-bool GetFstat(const char *path, struct stat *stbuf, tDlDesc *d=NULL)
+bool CheckLocalFile(const char *path, struct stat *stbuf)
 {
-	memcpy(stbuf, &statTempl, sizeof(statTempl));
-	stbuf->st_mode &= ~S_IFMT; // delete mode flags and set them as needed
+	//ldbg("Is a file?!");
 
-	// assume that everything downloadable is a file and a directory is 404
+	if (stat((altPath + path).c_str(), stbuf))
+		return false;
 
-	rechecks::eFileKind type=rechecks::GetFiletype(path);
-	_cerr ( "type: " << type );
-	if (type==rechecks::FILE_PKG || type==rechecks::FILE_INDEX)
+	header h;
+	int r = h.LoadFromFile(altPath + path + ".head");
+	if (r > 0)
 	{
-		//ldbg("Is a file!");
-		stbuf->st_mode |= S_IFREG;
-		stbuf->st_size=0;
-
-		tDlDesc e;
-		if (!d)
-			d=&e;
-
-		if (!d->CheckOrExplore(path))
-			return false;
-		if (d->seen_status!=200)
-			return false;
-		stbuf->st_size=d->seen_length;
+		const char *szLen = h.h[header::CONTENT_LENGTH];
+		if (szLen && *szLen && atoofft(szLen) == stbuf->st_size)
+		{
+			// ok, it's static and length is ok?
+			return true;
+		}
 	}
-	else
-	{
-		//ldbg("Is a directory!");
-		stbuf->st_mode |= S_IFDIR;
-		stbuf->st_size=4;
-	}
-	return true;
-	
+
+	return false;
 }
 
 
 /// If found as downloadable, present as a file, or become a directory otherwise.
 static int acngfs_getattr(const char *path, struct stat *stbuf)
 {
-	if(!GetFstat(path, stbuf))
+	// assume that everything downloadable is a file and a directory is 404
+
+	rechecks::eFileKind type = rechecks::GetFiletype(path);
+	_cerr( "type: " << type);
+	if (type == rechecks::FILE_PKG || type == rechecks::FILE_INDEX)
+	{
+		if (!altPath.empty() && type == rechecks::FILE_PKG && CheckLocalFile(path, stbuf))
+			return 0;
+
+		memcpy(stbuf, &statTempl, sizeof(statTempl));
+		stbuf->st_mode &= ~S_IFMT; // delete mode flags and set them as needed
+		stbuf->st_mode |= S_IFREG;
+		stbuf->st_size = 0;
+
+		tDlDescRemote e;
+		if (e.CheckOrExplore(path) && e.seen_status == 200)
+		{
+			stbuf->st_size = e.seen_length;
+			return 0;
+		}
+
 		return -EIO;
-	
-	return 0;
+	}
+	else
+	{
+		//ldbg("Is a directory!");
+		memcpy(stbuf, &statTempl, sizeof(statTempl));
+		stbuf->st_mode &= ~S_IFMT; // delete mode flags and set them as needed
+
+		stbuf->st_mode |= S_IFDIR;
+		stbuf->st_size = 4;
+		return 0;
+	}
 }
 
 static int acngfs_fgetattr(const char *path, struct stat *stbuf,
@@ -465,9 +595,21 @@ static int acngfs_open(const char *path, struct fuse_file_info *fi)
 			return -EROFS;
 
 	tDlDesc *p(NULL);
+	struct stat stbuf;
+
+	// ok... if that is a remote object, can we still use local access instead?
+	if(!altPath.empty() && rechecks::FILE_PKG == rechecks::GetFiletype(path)
+			 && CheckLocalFile(path, &stbuf))
+	{
+		p = tDlDescLocal::Create((altPath+path).c_str());
+		if(p)
+			goto desc_opened;
+	}
+
+
 	MYTRY
 	{
-		p=new tDlDesc;
+		p=new tDlDescRemote;
 		if(!p) // running exception-free? 
 			return -EIO;	
 	}
@@ -476,12 +618,24 @@ static int acngfs_open(const char *path, struct fuse_file_info *fi)
 		return -EIO;
 	}
 	
-	if(!p->CheckOrExplore(path) ||  p->seen_status !=200)
+	if( ! static_cast<tDlDescRemote*>(p)->CheckOrExplore(path) || p->seen_status !=200)
 	{
 		delete p;
+
+		// last alternative :-(
+		if(!altPath.empty())
+		{
+			p=tDlDescLocal::Create( (altPath+path).c_str());
+			if(p)
+				goto desc_opened;
+		}
+
 		return -EISDIR;
 	}
 
+
+	desc_opened:
+
 	/*
 	long size = p->h.h[header::CONTENT_LENGTH] ? atol(p->h.h[header::CONTENT_LENGTH]) : 0;  
 	if(size) // precache
@@ -546,14 +700,39 @@ static int acngfs_fsync(const char *path, int isdatasync,
 }
 
 
-struct fuse_operations acngfs_oper;
+struct fuse_operations_compat25 acngfs_oper;
+
+int my_fuse_main(int argc, char **argv)
+{
+#ifdef HAVE_DLOPEN
+   void *pLib = dlopen("libfuse.so.2", RTLD_LAZY);
+   if(!pLib)
+   {
+      cerr << "Couldn't find libfuse.so.2" <<endl;
+      return -1;
+   }
+   int (*myFuseMain) (int, char **, const struct fuse_operations_compat25 *, size_t);
+   *(void **) (&myFuseMain) = dlsym(pLib, "fuse_main_real_compat25");
+
+   if(!myFuseMain)
+   {
+	  cerr << "Error loading libfuse.so.2" <<endl;
+	  return -2;
+   }
+   return (*myFuseMain) (argc, argv, &acngfs_oper, sizeof(acngfs_oper));
+#else
+#warning dlopen not available
+   return fuse_main(argc, argv, &acngfs_oper);
+#endif
+}
 
 void _ExitUsage() {
    cerr << "USAGE: acngfs BaseURL ProxyHost MountPoint [FUSE Mount Options]\n"
-   << "example: acngfs http://ftp.uni-kl.de/debian localhost:3142 /var/local/aptfs\n"
-        << "valid FUSE Mount Options follow:\n";
+   << "examples:\n\t  acngfs http://ftp.uni-kl.de/debian cacheServer:3142 /var/local/aptfs\n"
+   << "\t  acngfs http://ftp.uni-kl.de/debian localhost:3142 /var/cache/apt-cacher-ng/debrep /var/local/aptfs\n\n"
+        << "FUSE mount options summary:\n\n";
     const char *argv[] = {"...", "-h"};
-    fuse_main( 2, const_cast<char**>(argv), &acngfs_oper);
+    my_fuse_main( 2, const_cast<char**>(argv));
     exit(EXIT_FAILURE);
 }
 
@@ -626,22 +805,32 @@ int main(int argc, char *argv[])
 	   exit(EXIT_FAILURE);
    }
    
+   unsigned int nMyArgCount = 2; // base url, proxy host
+   // alternative path supplied in the next argument?
+   if(argc > 4 && argv[4] && argv[4][0] != '-' ) // 4th argument is not an option?
+   {
+	   nMyArgCount=3;
+	   altPath = argv[3];
+   }
    
-   if(stat(argv[3], &statTempl) || statfs(argv[3], &stfsTemp))
-	   barf(endl << "Cannot access " << argv[3]);
+   // test mount point
+   const char *mpoint = argv[nMyArgCount+1];
+   if(stat(mpoint, &statTempl) || statfs(mpoint, &stfsTemp))
+	   barf(endl << "Cannot access " << mpoint);
    if(!S_ISDIR(statTempl.st_mode))
-      barf(endl<< argv[3] << " is not a directory.");
+      barf(endl<< mpoint << " is not a directory.");
 
+   ::rechecks::CompileExpressions();
 
-   // skip our arguments, keep those for fuse
-   argv[2]=argv[0]; // application path
-   argv=&argv[2];
-   argc-=2;
-   return fuse_main(argc, argv, &acngfs_oper);
+   // skip our arguments, keep those for fuse including mount point and argv[0] at the right place
+   argv[nMyArgCount]=argv[0]; // application path
+   argv=&argv[nMyArgCount];
+   argc-=nMyArgCount;
+   return my_fuse_main(argc, argv);
 }
 
 
-// for the uber-clever GNU linker and should be droped by strip again
+// for the uber-clever GNU linker and should be removed by strip again
 namespace aclog
 {
    void flush() {};
