@@ -51,20 +51,23 @@ struct tBackendDesc : public tHttpUrl
 };
 */
 
-class tRepoData : public MYSTD::vector<tHttpUrl>
+struct tRepoData
 {
-public:
+	MYSTD::vector<tHttpUrl> m_backends;
+
+	// dirty little helper to execute custom actions when a jobs associates or forgets this data set
 	struct IHookHandler
 	{
 		virtual void JobRelease()=0;
 		virtual void JobConnect()=0;
 	};
 	IHookHandler *m_pHooks;
-	tRepoData() : MYSTD::vector<tHttpUrl>(), m_pHooks(NULL) {};
+	tStrVec m_keyfiles;
+	tRepoData() : m_pHooks(NULL) {};
 	virtual ~tRepoData();
 };
 
-tRepoData * GetBackendVec(const mstring * vname);
+const tRepoData * GetBackendVec(const mstring * vname);
 
 time_t ExecutePostponed();
 
@@ -73,7 +76,7 @@ extern NoCaseStringMap mimemap;
 
 extern mstring cacheDirSlash; // guaranteed to have a trailing path separator
 
-}
+} // namespace acfg
 
 namespace rechecks
 {
@@ -86,6 +89,8 @@ eFileKind GetFiletype(const mstring &);
 bool MatchWhitelist(const mstring &);
 bool MatchUncacheableRequest(const mstring &);
 bool MatchUncacheableTarget(const mstring &);
+
+bool CompileExpressions();
 }
 
 #endif
