@@ -4,7 +4,7 @@
 
 #include "acsyscap.h"
 
-#define ACVERSION "0.6.3"
+#define ACVERSION "0.6.5"
 
 #define __STDC_FORMAT_MACROS
 #include <inttypes.h>
@@ -58,5 +58,8 @@
 
 #define ENEMIESOFDOSFS "?[]\\=+<>:;#"
 
+//! Time after which the pooled sockets are considered EOLed
+#define TIME_SOCKET_EXPIRE_CLOSE 33
+
 #endif // __CONFIG_H
 
