@@ -11,6 +11,8 @@
 
 #ifdef DEBUG
 
+#if 1 // def LOCAL_DEBUG
+
 #include <meta.h>
 #include <assert.h>
 #define ASSERT(x) { if(acfg::debug > SPAMLEVEL*2) assert(x); }
@@ -29,6 +31,21 @@
 
 #define dbgline { ldbg("mark" /*__FILE__ << ": " << __LINE__*/); }
 
+#else // LOCAL_DEBUG
+
+#define ldbgvl(v, x)
+#define dbglvl(v)
+#define ldbg(x)
+#define dbgline
+#define ASSERT(x)
+#define USRDBG(v, x)
+#define LOG(x)
+#define LOGSTART(x)
+#define LOGSTART2(x,y)
+
+#endif
+
+
 #elif defined(NOLOGATALL)
 
 #define ldbgvl(v, x)
