@@ -27,15 +27,16 @@
 
 #define EXTREME_MEMORY_SAVING false
 
-#define mstring MYSTD::string
-#define cmstring const MYSTD::string
-#define citer const_iterator
+typedef MYSTD::string mstring;
+typedef const MYSTD::string cmstring;
 
-#define tStrPos mstring::size_type 
-#define stmiss mstring::npos
-#define UINT unsigned int
-#define USHORT unsigned short
-#define UCHAR unsigned char
+typedef mstring::size_type tStrPos;
+const static tStrPos stmiss(cmstring::npos);
+typedef unsigned int UINT;
+typedef unsigned short USHORT;
+typedef unsigned char UCHAR;
+
+#define citer const_iterator
 
 #define CPATHSEPUNX '/'
 #define SZPATHSEPUNIX "/"
@@ -138,6 +139,9 @@ inline void trimString(mstring &s, const char *junk=SPACECHARS)
 		0==(where).compare((where).size()-(sizeof((what))-1), (sizeof((what))-1), (what)))
 #define stripSuffix(where, what) if(endsWithSzAr(where, what)) where.erase(where.size()-sizeof(what)+1);
 
+#define setIfNotEmpty(where, cand) { if(where.empty() && !cand.empty()) where = cand; }
+#define setIfNotEmpty2(where, cand, alt) { if(where.empty()) { if(!cand.empty()) where = cand; else where = alt; } }
+
 mstring GetBaseName(cmstring &in);
 mstring GetDirPart(cmstring &in);
 
@@ -148,6 +152,8 @@ tStrPos findHostStart(const mstring & sUri);
 #define _countof(x) sizeof(x)/sizeof(x[0])
 #endif
 
+#define NAMEWLEN(x) x, (_countof(x)-1)
+
 //extern mstring sPathSep, sPathSepUnix, sCR, sCRLF;
 
 // there is memchr and strpbrk but nothing like the last one acting on specified RAW memory range
@@ -361,5 +367,7 @@ off_t GetFileSize(cmstring & path, off_t defret);
 
 extern cmstring sEmptyString;
 
+#define UFI16_MAX (std::numeric_limits<uint_fast16_t>::max())
+
 #endif // _META_H
 
