@@ -8,24 +8,27 @@
 #ifndef TCPCONNECT_H_
 #define TCPCONNECT_H_
 
+class tcpconnect;
+typedef SHARED_PTR<tcpconnect> tTcpHandlePtr;
+
 class tcpconnect
 {
 public:
 	virtual ~tcpconnect();
 
-	static SHARED_PTR<tcpconnect> Create(cmstring &sHostnameHint);
-	static SHARED_PTR<tcpconnect> CreateConnected(cmstring &sHostname, mstring &sErrOut);
+	static SHARED_PTR<tcpconnect> CreateConnected(cmstring &sHostname, cmstring &sPort,
+			mstring &sErrOut,
+			bool *pbSecondHand=NULL, acfg::tRepoData::IHookHandler *pStateTracker=NULL);
 	/// Moves the connection handle to the reserve pool (resets the specified sptr).
 	/// Shold only be supplied with IDLE connection handles in a sane state.
-	static void RecycleIdleConnection(SHARED_PTR<tcpconnect> & handle);
+	static void RecycleIdleConnection(tTcpHandlePtr & handle);
 	static time_t ExpireCache();
 
+	bool Connect(cmstring &sHostname, cmstring &sPort, mstring &sErrOut);
 	inline int GetFD() { return m_conFd; }
 	inline cmstring & GetHostname() { return m_sHostName; }
-	inline void SetStateObserver(acfg::tRepoData::IHookHandler *pObs) { m_pConnStateObserver=pObs;}
+	inline cmstring & GetPort() { return m_sPort; }
 	const tHttpUrl * GetProxyData() const { return m_proxy; }
-
-	bool Connect(const mstring & sHostname, mstring &sErrOut);
 	void Disconnect();
 
 protected:
@@ -34,14 +37,12 @@ protected:
 	tcpconnect();
 
 	int m_conFd;
-	mstring m_sHostName;
+	mstring m_sHostName, m_sPort;
 
 	tHttpUrl * m_proxy;
-	time_t m_nExpTime;
 
 	acfg::tRepoData::IHookHandler *m_pConnStateObserver;
 };
 
-typedef SHARED_PTR<tcpconnect> tTcpHandlePtr;
 
 #endif /* TCPCONNECT_H_ */
