@@ -15,8 +15,9 @@ bool acbuf::init(unsigned int c) {
 	}
 	else
 	{
-		free(m_buf);
-		m_buf=(char*) malloc(c+1);
+		//free(m_buf);
+		//m_buf=(char*) malloc(c+1);
+		m_buf = (char*) realloc(m_buf, c+1);
 		if(!m_buf)
 			return false;
 		m_nCapacity=c;
