@@ -1,6 +1,6 @@
 
 
-#define LOCAL_DEBUG
+//#define LOCAL_DEBUG
 #include "debug.h"
 
 #include "acfg.h"
@@ -178,50 +178,17 @@ inline bool _ReadMainConfiguration(const string & szFilename)
 	return true;
 }
 
-inline const string * _CheckBEentryGetNamePtr(const string & sRepName)
+inline const string * GetRepoEntryNamePtr(const string & sRepName)
 {
-	// some memory saving using STL containers properties:
-	// needs a reliably stored string for the pointer. Backend description may not exist,
-	// then create a dummy one with no contents and point at this string. Iterators on map are
-	// guaranteed to be stable so point at its key variable.
+	// save some memory by storing only pointer to its entry in the map since the life time is guaranteed
 
-	pair<tRepMap::iterator,bool> addRes = repoparms.insert(make_pair(sRepName, tRepoData()));
-	return & addRes.first->first;
-}
-
-inline void _AddRemapInfo(bool bAsBackend, const string & token,
-		const string &repname)
-{
-	if (0!=token.compare(0, 5, "file:"))
+	tRepMap::iterator it = repoparms.find(sRepName);
+	if(repoparms.end() == it)
 	{
-		tHttpUrl url;
-		if(! url.SetHttpUrl(token))
-			BARF(token + " <-- bad URL detected");
-		_FixPostPreSlashes(url.sPath);
-		
-		if (bAsBackend)
-			repoparms[repname].push_back(url);
-		else
-			mapUrl2pVname.insert(pair<tHttpUrl,const string*>(
-					url, _CheckBEentryGetNamePtr(repname)));
-	}
-	else
-	{
-		string sPath=token.substr(5);
-		if (sPath.empty())
-			BARF("Bad file spec for repname, file:?");
-		
-		MakeAbsolutePath(sPath, confdir);
-		
-		tStrDeq srcs=ExpandFilePattern(sPath, true);
-		for(tStrDeq::const_iterator it=srcs.begin(); it!=srcs.end(); it++)
-		{
-			if (bAsBackend)
-				_ReadBackendsFiles(*it, repname);
-			else
-				_ReadRewriteFiles(*it, repname);
-		}
+		repoparms[sRepName] = tRepoData();
+		it = repoparms.find(sRepName);
 	}
+	return & it->first;
 }
 
 inline bool ParseOptionLine(const string &sLine, string &key, string &val, bool bQuiet)
@@ -253,6 +220,62 @@ inline bool ParseOptionLine(const string &sLine, string &key, string &val, bool
 }
 
 
+inline void AddRemapFlag(const string & token, const string &repname)
+{
+	mstring key, value;
+	if(!ParseOptionLine(token, key, value, true))
+		return;
+
+	if(key=="keyfile" && !value.empty())
+	{
+		//if(!startsWithSz(value, "/"))
+		//	value.insert(0, "/");
+		if (acfg::debug>3)
+			cerr << "Fatal keyfile for " <<repname<<": "<<value <<endl;
+
+		tRepoData &where = repoparms[repname];
+		where.m_keyfiles.push_back(value);
+	}
+}
+
+inline void AddRemapInfo(bool bAsBackend, const string & token,
+		const string &repname)
+{
+	if (0!=token.compare(0, 5, "file:"))
+	{
+		tHttpUrl url;
+		if(! url.SetHttpUrl(token))
+			BARF(token + " <-- bad URL detected");
+		_FixPostPreSlashes(url.sPath);
+		
+		if(url.sPort.empty())
+			url.sPort = remoteport;
+
+		if (bAsBackend)
+			repoparms[repname].m_backends.push_back(url);
+		else
+			mapUrl2pVname.insert(pair<tHttpUrl,const string*>(
+					url, GetRepoEntryNamePtr(repname)));
+	}
+	else
+	{
+		string sPath=token.substr(5);
+		if (sPath.empty())
+			BARF("Bad file spec for repname, file:?");
+		
+		MakeAbsolutePath(sPath, confdir);
+		
+		tStrDeq srcs=ExpandFilePattern(sPath, true);
+		for(tStrDeq::const_iterator it=srcs.begin(); it!=srcs.end(); it++)
+		{
+			if (bAsBackend)
+				_ReadBackendsFiles(*it, repname);
+			else
+				_ReadRewriteFiles(*it, repname);
+		}
+	}
+}
+
 struct tHookHandler: public tRepoData::IHookHandler, public lockable
 {
 	string cmdRel, cmdCon;
@@ -335,8 +358,8 @@ string _GetBase64Auth(const string & s)
 	{
 		c=s[pos++];
 		if('%' == c && pos<s.length()-1
-				&& hexmap[(unsigned)s[pos]]>=0
-				&& hexmap[(unsigned)s[pos+1]]>=0)
+				&& hexmap[(unsigned)s[pos]]<=15
+				&& hexmap[(unsigned)s[pos+1]]<=15)
 		{
 			c=char(16*hexmap[(unsigned)s[pos]]+hexmap[(unsigned)s[pos+1]]);
 			pos+=2;
@@ -526,22 +549,22 @@ bool SetOption(const string &sLine, bool bQuiet)
 				cerr << "Found invalid entry, ignoring " << key << ": " << value <<endl;
 			return false;
 		}
-		enum xtype { prefix, backend, flags } type = prefix;
-		for(UINT i=0; i<tokens.size(); i++)
+		int type(0); // prefixes ; backends ; flags
+		for(tStrVecIterConst it=tokens.begin(); it!=tokens.end(); ++it)
 		{
-			if(tokens[i]==";")
-			{
-				type=xtype(int(type)+1);
+			if(it->empty())
 				continue;
-			}
-			if(startsWithSz(tokens[i], "#"))
+			if(it->at(0)=='#')
 				break;
-			if(flags == type)
-			{
-#warning do something with the stuff
-			}
-			else
-				_AddRemapInfo(backend == type, tokens[i], vname);
+
+			if(it->at(0)==';')
+				++type;
+			else if(0 == type)
+				AddRemapInfo(false, *it, vname);
+			else if(1 == type)
+				AddRemapInfo(true, *it, vname);
+			else if(2 == type)
+				AddRemapFlag(*it, vname);
 		}
 		_AddHooksFile(vname);
 	}
@@ -590,12 +613,12 @@ const string * GetRepNameAndPathResidual(const tHttpUrl & in, string & sRetPathR
 	
 }
 
-tRepoData * GetBackendVec(const string * vname)
+const tRepoData * GetBackendVec(const string * vname)
 {
 	if(!vname)
 		return NULL;
 	tRepMap::iterator it=repoparms.find(*vname);
-	if(it==repoparms.end() || it->second.empty())
+	if(it==repoparms.end() || it->second.m_backends.empty())
 		return NULL;
 	return & it->second;
 }
@@ -641,7 +664,10 @@ void _ReadBackendsFiles(const string & sFile, const string &sRepName)
 			cerr << "Backend: " << sRepName << " <-- " << entry.ToURI() <<endl;
 #endif		
 
-			repoparms[sRepName].push_back(entry);
+			if(entry.sPort.empty())
+				entry.sPort = remoteport;
+
+			repoparms[sRepName].m_backends.push_back(entry);
 			nAddCount++;
 			entry.clear();
 		}
@@ -715,7 +741,7 @@ void _ReadRewriteFiles(const string & sFile, const string & sRepName)
 			{
 				_FixPostPreSlashes(url.sPath);
 				pair<tHttpUrl, const string*> info(url,
-						_CheckBEentryGetNamePtr(sRepName));
+						GetRepoEntryNamePtr(sRepName));
 				mapUrl2pVname.insert(info);
 			}
 			else
@@ -753,7 +779,7 @@ void _ReadRewriteFiles(const string & sFile, const string & sRepName)
 					tHttpUrl url;
 					url.sHost=*itHost;
 					url.sPath=*itPath;
-					pair<tHttpUrl,const string*> info(url, _CheckBEentryGetNamePtr(sRepName));
+					pair<tHttpUrl,const string*> info(url, GetRepoEntryNamePtr(sRepName));
 					mapUrl2pVname.insert(info);
 
 #ifdef DEBUG
@@ -1032,17 +1058,18 @@ time_t ExecutePostponed()
 
 namespace rechecks
 {
-	cmstring *psRex[] = {&acfg::vfilepat, &acfg::pfilepat, &acfg::wfilepat};
-	regex_t rex[_countof(psRex)];
+	const UINT rexCount = 3;
+	regex_t rex[rexCount];
 
 bool CompileExpressions()
 {
-	char buf[1024];
-	for(UINT i=0; i<_countof(psRex); i++)
+	cmstring *psRex[rexCount] = {&acfg::vfilepat, &acfg::pfilepat, &acfg::wfilepat};
+	for(UINT i=0; i<rexCount; i++)
 	{
 		int nErr=regcomp(&rex[i], psRex[i]->c_str(), REG_EXTENDED);
 		if(nErr)
 		{
+			char buf[1024];
 			regerror(nErr,  &rex[i], buf, sizeof(buf));
 			buf[_countof(buf)-1]=0; // better be safe...
 			std::cerr << buf << ": " << *(psRex[i]) << std::endl;
