@@ -26,10 +26,51 @@ void check_algos();
 
 int main(int argc, char **argv) {
 
+
+// PLAYGROUND
+
+/*
+ * Let's be another csum tool...
+
+	md5_state_s ctx;
+	md5_init(&ctx);
+	uint8_t buf[2000];
+	while(!feof(stdin))
+	{
+		int n=fread(buf, sizeof(char), 2000, stdin);
+		md5_append(&ctx, buf, n);
+	}
+	uint8_t csum[16];
+	md5_finish(&ctx, csum);
+	for(int i=0;i<16;i++)
+		printf("%02x", csum[i]);
+	printf("\n");
+	exit(0);
+*/
+	/*
+	uint8_t csum[16];
+	string s(argv[1]);
+	filereader::GetMd5Sum(s, csum);
+	for(int i=0;i<16;i++)
+			printf("%02x", csum[i]);
+		printf("\n");
+		if(CsEqual((unsigned char*) argv[2], csum))
+			printf("Is OK\n");
+		else
+			printf("Differenz\n");
+		exit(0);
+	*/
+	/*
+	bool Bz2compressFile(const char *, const char*);
+	return ! Bz2compressFile(argv[1], argv[2]);
+	*/
+
+
 	check_algos();
 	struct sigaction act;
 	memset(&act, 0, sizeof(act));
 
+	sigfillset(&act.sa_mask);
 	act.sa_handler = &term_handler;
 	sigaction(SIGTERM, &act, NULL);
 	sigaction(SIGINT, &act, NULL);
@@ -194,38 +235,3 @@ void term_handler(int signum)
 }
 
 
-#ifdef PLAYGROUND
-
-/* 
- * Let's be another csum tool...
- 
-	md5_state_s ctx;
-	md5_init(&ctx);
-	uint8_t buf[2000]; 
-	while(!feof(stdin))
-	{
-		int n=fread(buf, sizeof(char), 2000, stdin);
-		md5_append(&ctx, buf, n);
-	}
-	uint8_t csum[16];
-	md5_finish(&ctx, csum);
-	for(int i=0;i<16;i++)
-		printf("%02x", csum[i]);
-	printf("\n");
-	exit(0);
-*/
-	/*
-	uint8_t csum[16];
-	string s(argv[1]);
-	filereader::GetMd5Sum(s, csum);
-	for(int i=0;i<16;i++)
-			printf("%02x", csum[i]);
-		printf("\n");
-		if(CsEqual((unsigned char*) argv[2], csum))
-			printf("Is OK\n");
-		else
-			printf("Differenz\n");
-		exit(0);
-	*/
-	
-#endif
