@@ -181,7 +181,7 @@ void tWuiBgTask::Run(const string &cmd)
 			SendFmt() << "Maintenance task " << m_sTypeName
 					<< ", apt-cacher-ng version: " ACVERSION
 					" (<a href=" << cmd <<
-					"&sigabort>Cancel</a>)<br><form action=\"#top\">\n";
+					"&sigabort>Cancel</a>)<br><form id=\"mainForm\" action=\"#top\">\n";
 
 			Action(cmd);
 
@@ -193,9 +193,13 @@ void tWuiBgTask::Run(const string &cmd)
 			{
 				SendChunk("<hr><b>Action(s):</b> "
 					"<input type=\"submit\" name=\"doDelete\""
-					" value=\"Delete selected files\"><hr>");
+					" value=\"Delete selected files\">"
+					"|<button type=\"button\" onclick=\"checkOrUncheck(true);\">Check all</button>"
+					"<button type=\"button\" onclick=\"checkOrUncheck(false);\">Uncheck all</button><hr>");
 			}
 
+
+
 			_AddFooter(msg);
 			SendChunk(msg);
 	}
