@@ -1,5 +1,5 @@
 
-#define LOCAL_DEBUG
+//#define LOCAL_DEBUG
 #include "debug.h"
 
 #include "meta.h"
@@ -108,7 +108,7 @@ public:
 			return false;
 
 		string sErr;
-		m_spOutCon = tcpconnect::CreateConnected("bugs.debian.org", sErr);
+		m_spOutCon = tcpconnect::CreateConnected("bugs.debian.org", "80", sErr);
 
 		if(!m_spOutCon)
 			return false;
@@ -155,22 +155,7 @@ public:
 				{
 					serverBufOut.move();
 					char *p = 0;
-#warning FIXME
-          /*
-           * ==2308== Thread 2:
-==2308== Conditional jump or move depends on uninitialised value(s)
-==2308==    at 0x4C25EE1: memchr (mc_replace_strmem.c:446)
-==2308==    by 0x61BB629: __GI_strstr (str-two-way.h:269)
-==2308==    by 0x42673B: SoapPassThrough::TrySoapPassThrough(acbuf&, int) (conn.cc:158)
-==2308==    by 0x425528: con::WorkLoop() (conn.cc:273)
-==2308==    by 0x4273F1: conserver::ThreadAction(void*) (conserver.cc:71)
-==2308==    by 0x4E2F8B9: start_thread (pthread_create.c:300)
-==2308==    by 0x62063CC: clone (clone.S:112)
-==2308== 
-
-
-           *
-           * */
+					serverBufOut.flush();
 					while (0 != (p = strstr((char*)serverBufOut.rptr(), POSTMARK)))
 						memcpy(p, FAKEMARK, _countof(FAKEMARK) - 1);
 				}
@@ -238,7 +223,7 @@ void con::WorkLoop() {
         
         if(ready == 0)
         {
-        	USRDBG(5, "Timeout occured, apt client disappeared silently?");
+        	USRDBG(5, "Timeout occurred, apt client disappeared silently?");
         	return;
         }
 		else if (ready<0)
