@@ -136,7 +136,7 @@ void SetupConAndGo(int fd, const char *szClientName=NULL)
 			goto failure_mode;
 		}
 	}
-	MYCATCH(bad_alloc)
+	MYCATCH(MYSTD::bad_alloc&)
 	{
 		USRDBG(4, "Out of memory");
 		goto failure_mode;
@@ -252,7 +252,7 @@ void Setup()
           // bound, or maybe not, and then maybe because of the dual-behaviour,
           // or maybe because of real errors; we just cannot know for sure but
           // we have to.
-#ifdef IPV6_V6ONLY
+#if defined(IPV6_V6ONLY) && defined(SOL_IPV6)
 		    	if(p->ai_family==AF_INET6)
 		    		setsockopt(sockip, SOL_IPV6, IPV6_V6ONLY, &yes, sizeof(yes));
 #endif		    	
