@@ -149,7 +149,7 @@ bool tCacheMan::RefetchFile(tFileItemPtr fi, cmstring &sFilePathRel, mstring &sE
 	// abusing url class to extract base directory
 
 	tHttpUrl url;
-	acfg::tRepoData *pBackends(NULL);
+	const acfg::tRepoData *pBackends(NULL);
 
 	if (!url.SetHttpUrl(sFilePathRel))
 	{
@@ -221,7 +221,6 @@ bool tCacheMan::DownloadIdxFile(const MYSTD::string & sFilePathRel, string &sErr
 {
 	sErr.clear();
 	bool bSuccess=false;
-	FiStatus fistate=FIST_FRESH;
 	tFileItemPtr fi;
 
 #ifdef DEBUG
@@ -239,7 +238,7 @@ bool tCacheMan::DownloadIdxFile(const MYSTD::string & sFilePathRel, string &sErr
 
 	if (fi)
 	{
-		fistate = fi->Setup(true);
+		fi->Setup(true);
 		if(m_bForceDownload) // setup first and drop the contents then
 		{
 			fi->ResetCacheState();
@@ -1389,9 +1388,12 @@ bool tCacheMan::ParseAndProcessIndexFile(ifileprocessor &ret, const MYSTD::strin
 
 	if (!reader.OpenFile(CACHE_BASE+sPath))
 	{
-		aclog::errnoFmter err;
-		SendFmt()<<"<font color=orange>WARNING: unable to open "<<sPath<<"("
-				<< err << ")</font><br>\n";
+		if(! GetFlags(sPath).forgiveDlErrors) // that would be ok (added by ignorelist), don't bother
+		{
+			aclog::errnoFmter err;
+			SendFmt()<<"<font color=orange>WARNING: unable to open "<<sPath
+					<<"(" << err << ")</font><br>\n";
+		}
 		return false;
 	}
 
@@ -1735,8 +1737,11 @@ void tCacheMan::_ProcessSeenIndexFiles()
 
 		if( ! ParseAndProcessIndexFile(*this, it->first, itype))
 		{
-			SendChunk("<font color=red>An error occured while reading this file, some contents may have been ignored.</font><br>\n");
-			m_nErrorCount+=!m_indexFilesRel[it->first].forgiveDlErrors;
+			if(!m_indexFilesRel[it->first].forgiveDlErrors)
+			{
+				SendChunk("<font color=red>An error occured while reading this file, some contents may have been ignored.</font><br>\n");
+				m_nErrorCount++;
+			}
 			continue;
 		}
 		else if(!m_bNeedsStrictPathsHere && att.bros)
@@ -1860,9 +1865,12 @@ void expiration::HandlePkgEntry(const tRemoteFileInfo &entry, bool bUnpackForCsu
 			}
 			else
 			{
-				if(!GetFlags(sPathRel).parseignore)
+				tIfileAttribs at = GetFlags(sPathRel);
+				if(!at.parseignore && !at.forgiveDlErrors)
+				{
 					SendFmt()<<"<font color=\"orange\">WARNING, header file missing for "
 						<<sPathRel<<"</font><br>\n";
+				}
 			}
 		}
 
