@@ -1,5 +1,5 @@
 
-#define LOCAL_DEBUG
+//#define LOCAL_DEBUG
 #include "debug.h"
 
 #include "config.h"
@@ -86,21 +86,24 @@ fileitem::fileitem(MYSTD::string sPath) :
 	//ldbg("item created, m_sPath: "<< m_sPath);
 }
 
-void fileitem::AddDownloaderRef()
+void fileitem::IncDlRefCount()
 {
 	setLockGuard;
 	m_nDlRefsCount++;
 }
 
-void fileitem::DelDownloaderRef(const string &sReason)
+void fileitem::DecDlRefCount(const string &sReason)
 {
 	setLockGuard;
 	
+	notifyAll();
+
 	m_nDlRefsCount--;
 	if(m_nDlRefsCount>0)
 		return; // someone will care...
 	
-	notifyAll();
+	// ... otherwise: the last downloader disappeared, need to tell observers
+
 	if (status<FIST_COMPLETE)
 	{
 		status=FIST_ERROR;
@@ -474,6 +477,7 @@ bool fileitem::DownloadStartedStoreHeader(const header & h, const char *pNextDat
 		forceclose(m_filefd);
 	}
 
+	LOG("Deleting " << m_sPath);
 	unlink(m_sPath.c_str());
 	unlink(sHeadPath.c_str());
 	
@@ -625,6 +629,7 @@ tFileItemPtr fileitem::GetFileItem(MYSTD::string sPathKey)
 			p.reset();
 		return p;
 	}
+	return p;
 }
 
 
