@@ -25,6 +25,12 @@
 
 #define BUFSIZE 20000
 
+#ifdef MINIBUILD
+#undef HAVE_LIBBZ2
+#undef HAVE_ZLIB
+#undef HAVE_LZMA
+#endif
+
 using namespace MYSTD;
 
 filereader::filereader() 
@@ -293,7 +299,9 @@ inline void filereader::_UncompressMoreData() {
 		return;
 	}
 	
+#ifndef MINIBUILD
 	unsigned int nFeedLen=m_nBufSize-m_nBufPos;
+#endif
 	
 #ifdef HAVE_LIBBZ2
 	if(flags&FLAG_BZ2)
@@ -429,7 +437,7 @@ bool filereader::GetSha1Sum(const MYSTD::string & sFileName, uint8_t out[],
 }
 */
 
-
+#ifndef MINIBUILD
 class csumSHA1 : public csumBase, public SHA_INFO
 {
 public:
@@ -541,7 +549,8 @@ foreach $b (0..255) {
 print "\n";
 */
 
-#define _inv (uint_fast16_t)-1
+#define _inv UFI16_MAX
+
 uint_fast16_t hexmap[] = {
                 _inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,
                 _inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,_inv,
@@ -566,9 +575,9 @@ bool CsAsciiToBin(const char *a, uint8_t b[], unsigned short binLength)
 	const unsigned char *uA = (const unsigned char*) a;
 	for(int i=0; i<binLength;i++)
 	{
-		uint_fast16_t r=hexmap[uA[i*2]] * 16 + hexmap[uA[i*2+1]];
-		if(r>255) return false;
-		b[i]=r;
+		if(hexmap[uA[i*2]]>15 || hexmap[uA[i*2+1]] > 15)
+			return false;
+		b[i]=hexmap[uA[i*2]] * 16 + hexmap[uA[i*2+1]];
 	}
 	return true;
 }
@@ -604,6 +613,10 @@ bool filereader::CheckMd5Sum(const string & sFileName, const string & sReference
 
 */
 
+#endif
+
+#ifdef HAVE_LIBBZ2
+
 bool Bz2compressFile(const char *pathIn, const char*pathOut)
 {
 	bool bRet=false;
@@ -639,3 +652,5 @@ bool Bz2compressFile(const char *pathIn, const char*pathOut)
 	}
 	return bRet;
 }
+
+#endif
