@@ -1,5 +1,5 @@
 
-#define LOCAL_DEBUG
+//#define LOCAL_DEBUG
 #include "debug.h"
 
 #include "job.h"
@@ -52,8 +52,8 @@ public:
 	};
 	virtual FiStatus Setup(bool)
 	{
-		return status = FIST_INITED;
 		m_nSizeChecked = m_nSizeSeen = 0;
+		return status = FIST_INITED;
 	}
 	virtual void Unreg() {
 		setLockGuard;
@@ -78,11 +78,14 @@ public:
 	{
 		setLockGuard;
 
+		LOGSTART2("tPassThroughFitem::StoreFileData", "status: " << status);
+
 		// something might care, most likely... also about BOUNCE action
 		notifyAll();
 
 		m_nIncommingCount += size;
 
+		dbgline;
 		if (status >= FIST_ERROR || status < FIST_DLGOTHEAD)
 			return false;
 
@@ -90,6 +93,7 @@ public:
 			status = FIST_COMPLETE;
 		else
 		{
+			dbgline;
 			status = FIST_DLRECEIVING;
 			m_nSizeChecked += size;
 			m_pData = data;
@@ -98,10 +102,12 @@ public:
 			while(0 == m_nConsumed && status<FIST_ERROR)
 				wait();
 
+			dbgline;
 			// let the downloader abort?
 			if(status >= FIST_ERROR)
 				return false;
 
+			dbgline;
 			m_nConsumable=0;
 			m_pData=NULL;
 			return m_nConsumed;
@@ -479,7 +485,7 @@ void job::PrepareDownload() {
     string sRawUriPath, sPathResdiual;
     tHttpUrl tUrl; // parsed URL
         
-    acfg::tRepoData * pBackends(NULL); // appropriate backends
+    const acfg::tRepoData * pBackends(NULL); // appropriate backends
     const string * psVname(NULL); // virtual name for storage pool, if available
     
     FiStatus fistate(FIST_FRESH);
@@ -660,10 +666,13 @@ void job::PrepareDownload() {
 			if(psVname && NULL != (pBackends=acfg::GetBackendVec(psVname)))
 			{
 				LOG("Backends found, using them with " << sPathResdiual
-						<< ", first backend: " <<pBackends->front().ToURI());
+						<< ", first backend: " <<pBackends->m_backends.front().ToURI());
 
-				if(! bPtMode && rechecks::MatchUncacheableTarget(pBackends->front().ToURI()+sPathResdiual))
+				if(! bPtMode && rechecks::
+						MatchUncacheableTarget(pBackends->m_backends.front().ToURI()+sPathResdiual))
+				{
 					fistate=_SwitchToPtItem(m_sFileLoc);
+				}
 
 				m_pParentCon->m_pDlClient->AddJob(m_pItem, pBackends, sPathResdiual);
 			}
@@ -708,7 +717,7 @@ report_invpath:
     return ;
 
 report_invport:
-	SetErrorResponse("403 Invalid or prohibited port");
+	SetErrorResponse("403 Prohibited port (configure Remap-...)");
     return ;
 
 report_doubleproxy:
@@ -980,7 +989,11 @@ inline const char * job::BuildAndEnqueHeader(const FiStatus &fistate,
 			LOG("has contlen? " << bool(respHead.h[header::CONTENT_LENGTH]));
 		}
 		else if (httpstatus >= 300)
-			bHasSendableData = (respHead.h[header::CONTENT_LENGTH] || bChunked);
+		{
+			bHasSendableData = ( (respHead.h[header::CONTENT_LENGTH]
+			                                 && atoi(respHead.h[header::CONTENT_LENGTH])>0)
+					|| bChunked);
+		}
 	}
 
 	tSS &sb=m_sendbuf;
@@ -1109,7 +1122,11 @@ inline const char * job::BuildAndEnqueHeader(const FiStatus &fistate,
 			sb<<"application/octet-stream\r\n";
 	}
 	else
+	{
+		sb<<"Content-Length: 0\r\n";
+
 		m_backstate=STATE_ALLDONE;
+	}
 
 	sb << header::GenInfoHeaders();
 
