@@ -7,6 +7,7 @@
 #include "expiration.h"
 #include "pkgimport.h"
 #include "showinfo.h"
+#include "mirror.h"
 #include "aclogger.h"
 #include "filereader.h"
 #include "acfg.h"
@@ -25,17 +26,19 @@ using namespace MYSTD;
 
 #define MAINT_HTML_DECO "maint.html" 
 
-maintenance::maintenance(int fd) :
+tWUIPage::tWUIPage(int fd) :
 	m_reportFD(fd),
-	m_szDecoFile(NULL)
+	m_szDecoFile(NULL),
+	m_fmtBuf(NULL), m_fmtSize(0)
 {
 }
 
-maintenance::~maintenance()
+tWUIPage::~tWUIPage()
 {
+	free(m_fmtBuf);
 }
 
-bool maintenance::SendRawData(const char *data, size_t len, int flags)
+bool tWUIPage::SendRawData(const char *data, size_t len, int flags)
 {
 	while(len>0)
 	{
@@ -54,7 +57,7 @@ bool maintenance::SendRawData(const char *data, size_t len, int flags)
 	return true;
 }
 
-void maintenance::SendChunk(const char *data, size_t len)
+void tWUIPage::SendChunk(const char *data, size_t len)
 {
 	if(!data || !len || m_reportFD<0)
 		return;
@@ -66,12 +69,12 @@ void maintenance::SendChunk(const char *data, size_t len)
 	SendRawData("\r\n", 2, MSG_NOSIGNAL);
 }
 
-void maintenance::EndTransfer() 
+void tWUIPage::EndTransfer() 
 {
 	SendRawData(_SZ2PS("0\r\n\r\n"), MSG_NOSIGNAL);
 }
 
-void maintenance::SendChunkedPageHeader(const char *httpcode, const char *mimetype)
+void tWUIPage::SendChunkedPageHeader(const char *httpcode, const char *mimetype)
 {
 	tSS s(100);
 	s <<  "HTTP/1.1 " << (httpcode ? httpcode : "200") << " OK\r\n"
@@ -80,45 +83,11 @@ void maintenance::SendChunkedPageHeader(const char *httpcode, const char *mimety
 			"Content-Type: " << (mimetype?mimetype:"text/html") << "\r\n\r\n";
 	SendRawData(s.data(), s.length(), MSG_MORE);
 }
-/*
-void maintenance::SendDecoration(bool bBegin, const char *szDecoFile)
-{
 
-	acbuf m_deco;
-	char *mark(NULL);
-	
-	if(szDecoFile && m_deco.initFromFile((acfg::confdir+sPathSep+szDecoFile).c_str()))
-		mark=strchr(m_deco.c_str(), '~');
-	
-	// deco file specified and loaded?
-	if(mark)
-	{
-		if(bBegin)
-			m_deco.erase(mark-m_deco.rptr());
-		else
-			m_deco.drop(mark-m_deco.rptr()+1);
-		string sDeco(m_deco.rptr(), m_deco.size());
-		SetStyle(sDeco);
-		SendMsg(sDeco);
-	}	
-	else
-		SendMsg(bBegin?"<html><body>":"</html></body>");
-	
-	if(!bBegin)
-		EndTransfer();
-}
-*/
-class authbounce : public maintenance
+class tAuthRequest : public tWUIPage
 {
 public:
-	// some NOOPs
-	authbounce(int fd) : maintenance(fd)
-	{
-	}
-	~authbounce()
-	{
-	}
-	void Action(const MYSTD::string & src)
+	tAuthRequest(int fd) : tWUIPage(fd)
 	{
 	}
 	void Run(const string &)
@@ -126,38 +95,127 @@ public:
 		const char authmsg[] = "HTTP/1.1 401 Not Authorized\r\nWWW-Authenticate: "
         "Basic realm=\"Apt-Cacher NG administration area\"\r\n"
         "Connection: Close\r\n"
-        "Content-Type: text/html\r\nContent-Length:81\r\n\r\n"
+        "Content-Type: text/plain\r\nContent-Length:81\r\n\r\n"
         "Not Authorized. Please contact Apt-Cacher NG administrator for further questions.";
 		SendRawData(authmsg, sizeof(authmsg)-1, 0);
 	}
-	
 };
 
-void DispatchAndRunMaintTask(const MYSTD::string &cmd, int conFD, const char * szAuthLine)
+class authbounce : public tWUIPage
+{
+public:
+	authbounce(int fd) : tWUIPage(fd)
+	{
+	}
+	void Run(const string &)
+	{
+		const char authmsg[] = "HTTP/1.1 200 Not Authorized\r\n"
+        "Connection: Close\r\n"
+        "Content-Type: text/plain\r\nContent-Length: 102\r\n\r\n"
+        "Not Authorized. To start this action, an administrator password must be set and "
+        "you must be logged in.";
+		SendRawData(authmsg, sizeof(authmsg)-1, 0);
+	}
+};
+
+string & tWUIPage::GetHostname()
+{
+	if (m_sHostname.empty())
+	{
+		struct sockaddr_storage ss;
+		socklen_t slen = sizeof(ss);
+		char hbuf[NI_MAXHOST];
+
+		if (0==getsockname(m_reportFD, (struct sockaddr *)&ss, &slen) && 0
+				==getnameinfo((struct sockaddr*) &ss, sizeof(ss), hbuf,
+						sizeof(hbuf),
+						NULL, 0, NI_NUMERICHOST))
+		{
+			const char *p=hbuf;
+			bool bAddBrs(false);
+			if(0==strncmp(hbuf, "::ffff:", 7) && strpbrk(p, "0123456789."))
+				p+=7; // no more colons there, looks like v4 IP in v6 space -> crop it
+			else if(strchr(p, (int) ':'))
+				bAddBrs=true; // full v6 address for sure, add brackets
+
+			if(bAddBrs)
+				m_sHostname="[";
+			m_sHostname+=p;
+			if(bAddBrs)
+				m_sHostname+="]";
+		}
+		else
+			m_sHostname="IP-of-this-cache-server";
+	}
+	return m_sHostname;
+}
+
+tWUIPage::tFmtSendObj::tFmtSendObj(tWUIPage *p) : m_parent(p)
+{
+	m_buf = m_parent->m_fmtBuf;
+	m_nCapacity = m_parent->m_fmtSize;
+}
+
+tWUIPage::tFmtSendObj::~tFmtSendObj()
+{
+	m_parent->SendChunk(data(), length());
+	m_parent->m_fmtBuf = m_buf;
+	m_parent->m_fmtSize = m_nCapacity;
+	m_buf=NULL;
+	m_nCapacity=0;
+}
+
+
+void DispatchAndRunMaintTask(MYSTD::string &cmd, int conFD, const char * szAuthLine)
 {	
-	maintenance *pWorker(NULL);
+	tWUIPage *pWorker(NULL);
 	if(!szAuthLine)
 		szAuthLine="";
 
+#ifdef DEBUG
+		if(cmd.find("tickTack")!=stmiss)
+		{
+			tBgTester(conFD).Run(cmd);
+			return;
+		}
+#endif
+
 	// not effective, why? signal(SIGPIPE, SIG_IGN);
 	
 	MYTRY
 	{
 		//std::cout << "vgl: " << authLine << " und " << acfg::adminauth << "\n";
-		
 		// admin actions are passed with GET parameters, appended after ?
-		if(cmd.find('?')!=stmiss)
+		tStrPos qpos=cmd.find('?');
+		reeval_cmd:
+		if(qpos!=stmiss)
 		{
 			if( ! acfg::adminauth.empty() && acfg::adminauth!=szAuthLine )
-				pWorker = new authbounce(conFD);
+				pWorker = new tAuthRequest(conFD);
 			else if(cmd.find("doExpire=")!=stmiss || 
 					cmd.find("justShow=")!=stmiss ||
 					cmd.find("justRemove=")!=stmiss)
 				pWorker = new expiration(conFD);
 			else if(cmd.find("doImport=")!=stmiss)
 				pWorker=new pkgimport(conFD);
+			else if(cmd.find("doMirror=")!=stmiss)
+				if(acfg::adminauth.empty())
+					pWorker=new authbounce(conFD);
+				else
+					pWorker=new pkgmirror(conFD);
+			else if(cmd.find("doDelete=")!=stmiss ||cmd.find("doDeleteYes=")!=stmiss)
+				if(acfg::adminauth.empty())
+					pWorker=new authbounce(conFD);
+				else
+					pWorker=new tDeleter(conFD);
 			else if(cmd.find("doCount=")!=stmiss)
 				pWorker = new tStaticFileSend(conFD, "report.html", "text/html", "200");
+			else // ok... just show the default page
+			{
+				cmd=cmd.substr(0,qpos);
+				qpos=stmiss;
+				goto reeval_cmd;
+			}
 		}
 		else if (cmd==acfg::reportpage)
 			pWorker = new tStaticFileSend(conFD, "report.html", "text/html", "200");
@@ -171,56 +229,5 @@ void DispatchAndRunMaintTask(const MYSTD::string &cmd, int conFD, const char * s
 	}
 	MYCATCH(...)
 	{ /* whatever */ };
-	if(pWorker)
-		delete pWorker;
+	delete pWorker;
 }
-
-void maintenance::SetStyle(string &s)
-{
-	tStrPos pos=0;
-
-	acbuf buf;
-	if( ! buf.initFromFile( (acfg::confdir+SZPATHSEP "style.css").c_str()) )
-		return;
-	
-	while(stmiss != (pos = s.find("$STYLE", pos)))
-	{
-		//cerr << "replacing " << s.substr(pos, 6) << " with " << buf.c_str()<<endl;
-		s.replace(pos, 6, buf.c_str());
-		pos+=buf.size();
-	}
-}
-
-string & maintenance::GetHostname()
-{
-	if (m_sHostname.empty())
-	{
-		struct sockaddr_storage ss;
-		socklen_t slen = sizeof(ss);
-		char hbuf[NI_MAXHOST];
-
-		if (0==getsockname(m_reportFD, (struct sockaddr *)&ss, &slen) && 0
-				==getnameinfo((struct sockaddr*) &ss, sizeof(ss), hbuf,
-						sizeof(hbuf), 
-						NULL, 0, NI_NUMERICHOST))
-		{
-			const char *p=hbuf;
-			bool bAddBrs(false);
-			if(0==strncmp(hbuf, "::ffff:", 7) && strpbrk(p, "0123456789."))
-				p+=7; // no more colons there, looks like v4 IP in v6 space -> crop it
-			else if(strchr(p, (int) ':'))
-				bAddBrs=true; // full v6 address for sure, add brackets
-			
-			if(bAddBrs)
-				m_sHostname="[";
-			m_sHostname+=p;
-			if(bAddBrs)
-				m_sHostname+="]";
-		}
-		else
-			m_sHostname="IP-of-this-cache-server";
-	}
-	return m_sHostname;
-
-}
-
