@@ -20,7 +20,10 @@
 
 using namespace MYSTD;
 
-tcpconnect::tcpconnect() :	m_conFd(-1), m_nExpTime(0), m_pConnStateObserver(NULL)
+//#warning FIXME, hack
+//#define NOCONCACHE
+
+tcpconnect::tcpconnect() :	m_conFd(-1), m_pConnStateObserver(NULL)
 {
 	m_proxy = acfg::proxy_info.sHost.empty() ? NULL : &acfg::proxy_info;
 }
@@ -93,13 +96,12 @@ static int connect_timeout(int sockfd, const struct sockaddr *addr, socklen_t ad
 	return 0;
 }
 
-
-bool tcpconnect::Connect(const string & sHostname, string & sErrorMsg)
+bool tcpconnect::Connect(const string & sHostname, const string & sPort, string & sErrorMsg)
 {
 	LOGSTART2("tcpconnect::_Connect", "hostname: " << sHostname);
 	CAddrInfo::SPtr dns = m_proxy ?
 			CAddrInfo::CachedResolve(m_proxy->sHost, m_proxy->sPort, sErrorMsg)
-			: CAddrInfo::CachedResolve(sHostname, acfg::remoteport, sErrorMsg);
+			: CAddrInfo::CachedResolve(sHostname, sPort, sErrorMsg);
 	if(!dns)
 	{
 		USRDBG(4, sErrorMsg);
@@ -136,13 +138,16 @@ bool tcpconnect::Connect(const string & sHostname, string & sErrorMsg)
 		ldbg("connect() ok");
 		set_nb(m_conFd);
 		m_sHostName = sHostname;
+		m_sPort = sPort;
 		return true;
 	}
 
+#ifdef MINIBUILD
+	sErrorMsg = "500 Connection failure";
+#else
 	// format the last available error message for the user
-	aclog::errnoFmter fer;
-	sErrorMsg="500 Connection failure: ";
-	sErrorMsg+=fer;
+	sErrorMsg=aclog::errnoFmter("500 Connection failure: ");
+#endif
 	ldbg("Force reconnect, con. failure");
 	Disconnect();
 	return false;
@@ -193,74 +198,126 @@ void termsocket(int fd)
 }
 
 lockable conPoolMx;
-/*
-bool lessByConnName(const tTcpHandlePtr &a, const tTcpHandlePtr &b)
+using namespace MYSTD;
+struct tHostHint // could derive from pair but prefer to save some bytes with references
 {
-	return a->GetHostname() < b->GetHostname();
-}
-typedef std::multiset<tTcpHandlePtr, lessByConnName>  tConpoolmap;
-*/
-MYSTD::multimap<cmstring, tTcpHandlePtr> spareConPool;
+	cmstring &pHost, &pPort;
+	inline tHostHint(cmstring &h, cmstring &p, time_t d=0) : pHost(h), pPort(p) {};
+	bool operator<(const tHostHint &b) const
+	{
+		int rel=pHost.compare(b.pHost);
+		return rel ? rel < 0 : pPort < b.pPort;
+	}
+};
+typedef multimap<tHostHint, std::pair<tTcpHandlePtr, time_t> > tConPool;
+tConPool spareConPool;
 
-tTcpHandlePtr tcpconnect::Create(cmstring &sHostnameHint)
+tTcpHandlePtr tcpconnect::CreateConnected(cmstring &sHostname, cmstring &sPort,
+		mstring &sErrOut, bool *pbSecondHand, acfg::tRepoData::IHookHandler *pStateTracker)
 {
-	lockguard __g(conPoolMx);
-	MYSTD::map<cmstring, tTcpHandlePtr>::iterator it=spareConPool.find(sHostnameHint);
-	if(spareConPool.end() == it)
-		return tTcpHandlePtr(new tcpconnect);
-	tTcpHandlePtr ret=it->second;
-	spareConPool.erase(it);
-	return ret;
-}
+	tTcpHandlePtr p;
+	bool bReused=false;
+	tHostHint key(sHostname, sPort);
 
-tTcpHandlePtr tcpconnect::CreateConnected(cmstring &sHostname, mstring &sErrOut)
-{
-	tTcpHandlePtr p = Create(sHostname);
-	if(!p || !p->Connect(sHostname, sErrOut) || p->GetFD()<0) // failed or worthless
+#ifdef NOCONCACHE
+	p.reset(new tcpconnect);
+	if(!p || !p->Connect(sHostname, sPort, sErrOut) || p->GetFD()<0) // failed or worthless
 		p.reset();
+#else
+	{
+		lockguard __g(conPoolMx);
+		tConPool::iterator it=spareConPool.find(key);
+		if(spareConPool.end() == it)
+		{
+			p.reset(new tcpconnect);
+			if(!p || !p->Connect(sHostname, sPort, sErrOut) || p->GetFD()<0) // failed or worthless
+				p.reset();
+		}
+		else
+		{
+			p=it->second.first;
+			spareConPool.erase(it);
+			bReused = true;
+		}
+	}
+#endif
+
+	if(p && pStateTracker)
+	{
+		p->m_pConnStateObserver = pStateTracker;
+		pStateTracker->JobConnect();
+	}
+
+	if(pbSecondHand)
+		*pbSecondHand = bReused;
+
 	return p;
 }
 
-void tcpconnect::RecycleIdleConnection(SHARED_PTR<tcpconnect> & handle)
+void tcpconnect::RecycleIdleConnection(tTcpHandlePtr & handle)
 {
 	if(!handle)
 		return;
 
+	if(handle->m_pConnStateObserver)
+	{
+		handle->m_pConnStateObserver->JobRelease();
+		handle->m_pConnStateObserver = NULL;
+	}
+
+#ifndef NOCONCACHE
 	time_t now(time(0));
 	lockguard __g(conPoolMx);
 
 	// a DOS?
 	if(spareConPool.size()<50)
 	{
-		handle->m_nExpTime=now+23;
-		spareConPool.insert(make_pair(handle->GetHostname(), handle));
+		spareConPool.insert(make_pair(tHostHint(handle->GetHostname(), handle->GetPort()),
+				make_pair(handle, now)));
 	}
+#endif
 
 	handle.reset();
 }
 
 
+
+
 time_t tcpconnect::ExpireCache()
 {
 	lockguard __g(conPoolMx);
-	time_t now(time(0)), ret(cleaner::never);
+	time_t now(time(0));
 
-	for(MYSTD::map<cmstring, tTcpHandlePtr>::iterator it=spareConPool.begin();
-			it!=spareConPool.end(); )
+	fd_set rfds;
+	FD_ZERO(&rfds);
+	int nMaxFd=0;
+
+	// either drop the old ones, or stuff them into a quick select call to find the good sockets
+	for (tConPool::iterator it = spareConPool.begin(); it != spareConPool.end();)
 	{
-		if(it->second)
+		if (now >= (it->second.second + TIME_SOCKET_EXPIRE_CLOSE))
+			spareConPool.erase(it++);
+		else
 		{
-			tcpconnect *p=it->second.get();
-			if(p->m_nExpTime<now)
-			{
-				spareConPool.erase(it++);
-				continue;
-			}
-			else
-				ret=min(ret, it->second->m_nExpTime);
+			int fd = it->second.first->GetFD();
+			FD_SET(fd, &rfds);
+			nMaxFd = max(nMaxFd, fd);
+			++it;
 		}
-
-		++it;
 	}
-	return ret;
+	// if they have to send something, that must the be the CLOSE signal
+	struct timeval tv;
+	tv.tv_sec = 0;
+	tv.tv_usec = 1;
+	int r=select(nMaxFd + 1, &rfds, NULL, NULL, &tv);
+	// on error, also do nothing, or stop when r fds are processed
+	for (tConPool::iterator it = spareConPool.begin(); r>0 && it != spareConPool.end(); r--)
+	{
+		if(FD_ISSET(it->second.first->GetFD(), &rfds))
+			spareConPool.erase(it++);
+		else
+			++it;
+	}
+
+	return spareConPool.empty() ? cleaner::never : time(0)+TIME_SOCKET_EXPIRE_CLOSE/4+1;
 }
